package cn.iocoder.yudao.module.product.dal.mysql.productbrank;

import java.util.*;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import cn.iocoder.yudao.framework.mybatis.core.mapper.AbstractMapper;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.product.dal.dataobject.productbrank.ProductBrankDO;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.product.vo.productbrandempower.GetFeeTypeProductBrandEmpowerReqVO;
import cn.iocoder.yudao.module.product.vo.productbrandempower.ProductLineBrandPriceVO;
import cn.iocoder.yudao.module.product.vo.productbrank.ProductBrankPageReqVO;
import cn.iocoder.yudao.module.product.vo.productbrank.ProductBrankRespVO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.apache.commons.lang3.StringUtils;
import org.apache.ibatis.annotations.Mapper;
import cn.iocoder.yudao.module.product.vo.productbrank.*;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.ResultType;
import org.apache.ibatis.annotations.Select;

/**
 * 品牌 Mapper
 *
 * @author 江楠
 */
@Mapper
public interface ProductBrankMapper extends AbstractMapper<ProductBrankDO> {
    @Override
    default PageResult<ProductBrankDO> selectPage(PageVO page, Object object) {
        if (object instanceof ProductBrankPageReqVO) {
            ProductBrankPageReqVO reqVO = (ProductBrankPageReqVO) object;
            return selectPage(page, new LambdaQuery<ProductBrankDO>()
                    .eqIfPresent(ProductBrankDO::getTitleZh, reqVO.getTitleZh())
                    .eqIfPresent(ProductBrankDO::getTitleEn, reqVO.getTitleEn())
                    .eqIfPresent(ProductBrankDO::getAorder, reqVO.getAorder())
                    .eqIfPresent(ProductBrankDO::getFiling, reqVO.getFiling())
                    .betweenIfPresent(ProductBrankDO::getCreateTime, reqVO.getBeginCreateTime(), reqVO.getEndCreateTime())
                    .orderByDesc(ProductBrankDO::getId));
        }

        return null;
    }

    @Override
    default List<ProductBrankDO> selectList(Object object) {
        if (object instanceof ProductBrankPageReqVO) {
            ProductBrankPageReqVO reqVO = (ProductBrankPageReqVO) object;
            return selectList(new LambdaQuery<ProductBrankDO>()
                    .eqIfPresent(ProductBrankDO::getTitleZh, reqVO.getTitleZh())
                    .eqIfPresent(ProductBrankDO::getTitleEn, reqVO.getTitleEn())
                    .eqIfPresent(ProductBrankDO::getAorder, reqVO.getAorder())
                    .eqIfPresent(ProductBrankDO::getFiling, reqVO.getFiling())
                    .betweenIfPresent(ProductBrankDO::getCreateTime, reqVO.getBeginCreateTime(), reqVO.getEndCreateTime())
                    .orderByDesc(ProductBrankDO::getId));
        } else if (object instanceof ProductBrankExportReqVO) {
            ProductBrankExportReqVO reqVO = (ProductBrankExportReqVO) object;
            return selectList(new LambdaQuery<ProductBrankDO>()
                    .eqIfPresent(ProductBrankDO::getTitleZh, reqVO.getTitleZh())
                    .eqIfPresent(ProductBrankDO::getTitleEn, reqVO.getTitleEn())
                    .eqIfPresent(ProductBrankDO::getAorder, reqVO.getAorder())
                    .eqIfPresent(ProductBrankDO::getFiling, reqVO.getFiling())
                    .betweenIfPresent(ProductBrankDO::getCreateTime, reqVO.getBeginCreateTime(), reqVO.getEndCreateTime())
                    .orderByDesc(ProductBrankDO::getId));
        }

        return null;
    }

    default PageResult<ProductBrankDO> selectPage(ProductBrankPageReqVO reqVO) {
        LambdaQueryWrapperX<ProductBrankDO> productBrankDOLambdaQueryWrapper = new LambdaQueryWrapperX<ProductBrankDO>();
//                .likeIfPresent(ProductBrankDO::getTitleZh, reqVO.getTitleZh())
        productBrankDOLambdaQueryWrapper.likeIfPresent(ProductBrankDO::getTitleEn, reqVO.getTitleEn())
                .eqIfPresent(ProductBrankDO::getAorder, reqVO.getAorder())
                .eqIfPresent(ProductBrankDO::getFiling, reqVO.getFiling())
                .betweenIfPresent(ProductBrankDO::getCreateTime, reqVO.getBeginCreateTime(), reqVO.getEndCreateTime());
        if (StringUtils.isNotEmpty(reqVO.getTitleZh())) {
            productBrankDOLambdaQueryWrapper.apply(" lower(title_zh) like concat('%', '" + reqVO.getTitleZh().toLowerCase()+ "','%')");
        }
        productBrankDOLambdaQueryWrapper.orderByDesc(ProductBrankDO::getId);
        return selectPage(reqVO, productBrankDOLambdaQueryWrapper);
    }

    /**
     * 查询商品是否在品牌商品范围内
     */
    @ResultType(List.class)
    @Select(
            {
                    "<script>",
                    "select ",
                    "a.id,",
                    "a.title_zh,",
                    "a.title_en,",
                    "a.aorder,",
                    "a.filing,",
                    "a.creator,",
                    "a.create_time,",
                    "a.updater,",
                    "a.update_time,",
                    "a.unauth_charging_model,",
                    "a.unauth_default_fee_type,",
                    "a.deleted",
                    "from ecw_product_brank a,ecw_product_brand_trademark_rel b,ecw_product_brand_trademark_cateogry_rel c",
                    "WHERE a.deleted = 0 ",
                    "and a.id = #{pageVO.brandId} ",
                    "and a.id = b.product_brand_id ",
                    "and b.id = c.trademark_id ",
                    "and ( ",
                    //全部商品
                    "c.product_category_id = 0",
                    //指定商品分类
                    "or (c.product_category_id = (select type_id from ecw_product where id = #{pageVO.productId} limit 1) and c.product_id = 0) ",
                    "or (c.product_category_id = (select type_id from ecw_product where id = #{pageVO.productId} limit 1) and c.product_id = #{pageVO.productId}) ",
                    ")",
                    " order by a.aorder desc limit 1 ",
                    "</script>"
            }
    )
    ProductBrankRespVO getBrandByProductBrand(@Param("pageVO") GetFeeTypeProductBrandEmpowerReqVO pageVO);


    @ResultType(List.class)
    @Select(
            {
                    "<script>",
                    "select ",
                    "a.id,",
                    "a.title_zh,",
                    "a.title_en,",
                    "a.aorder,",
                    "a.filing,",
                    "a.creator,",
                    "a.create_time,",
                    "a.updater,",
                    "a.update_time,",
                    "a.unauth_charging_model,",
                    "a.unauth_default_fee_type,",
                    "a.deleted",
                    "from ecw_product_brank a left join ecw_product_brand_trademark_rel b on a.id = b.product_brand_id ",
                    "left join ecw_product_brand_trademark_cateogry_rel c on b.id = c.trademark_id ",
                    "WHERE a.deleted = 0 ",
                    "and a.id = #{brandId} ",
                    "and ( ",
                    //全部商品
                    "c.product_category_id = 0",
                    //指定商品分类下全部商品
                    "or (c.product_category_id = (select type_id from ecw_product where id = #{productId} limit 1) and c.product_id = 0) ",
                    "or (c.product_category_id = (select type_id from ecw_product where id = #{productId} limit 1) and c.product_id = #{productId}) ",
                    "or c.product_category_id  is null",
                    ")",
                    " order by a.aorder desc limit 1 ",
                    "</script>"
            }
    )
    ProductBrankRespVO getBrandByProductBrandByParam(@Param("productId") Long productId, @Param("brandId") Long brandId);


    @ResultType(Long.class)
    @Select(
            {
                    "<script>",
                    "select ",
                    "customer_id from ecw_order",
                    "where 1=1",
                    "and order_id = #{orderId}",
                    " order by order_id desc limit 1 ",
                    "</script>"
            }
    )
    Long getCustomerIdByProductBrandByOrderId(@Param("orderId") Long orderId);

    @ResultType(Long.class)
    @Select("select customer_id from ecw_order_consignor where order_id = #{orderId} order by id desc limit 1 ")
    Long getNorCustomerIdByOrderId(@Param("orderId") Long orderId);

    @ResultType(Long.class)
    @Select("select customer_id from ecw_order_consignee where order_id = #{orderId} order by id desc limit 1 ")
    Long getNeeCustomerIdByOrderId(@Param("orderId") Long orderId);

    @ResultType(ProductLineBrandPriceVO.class)
    @Select({
            "<script>",
            "select ",
            "IFNULL(ps1.transport_price, 0) as have,",
            "IFNULL(ps2.transport_price, 0) as neutral",
            " from ecw_product_price p ",
            "left join ecw_product_price_special ps1 ",
            "on p.id = ps1.product_price_id and ps1.special_dict_type = 'licensed_price' ",
            "left join ecw_product_price_special ps2 ",
            "on p.id = ps2.product_price_id and ps2.special_dict_type = 'neutral_brand_prices' ",
            "where p.deleted = 0",
            "and (p.validate_start_date is null or p.validate_start_date <![CDATA[<=]]> now())",
            "and (p.validate_end_date is null or p.validate_end_date <![CDATA[>=]]> now())",
            "and p.product_id = #{prodId}",
            " and p.warehouse_line_id = #{lineId}",
            "<when test = 'channelId != null and channelId > 0 '>",
            " and p.shipping_channel_id = #{channelId}",
            "</when>",
            " order by p.id desc limit 1 ",
            "</script>"
    })
    ProductLineBrandPriceVO getProductLineBrandPrice(@Param("lineId") Long lineId, @Param("channelId") Long channelId, @Param("prodId") Long prodId);

    @ResultType(ProductLineBrandPriceVO.class)
    @Select("select IFNULL(o.line_id,0) as line_id, IFNULL(o.channel_id,0) as channel_id from ecw_order o where o.order_id = #{orderId} and o.deleted = 0")
    ProductLineBrandPriceVO getLineAndChannelByOrderId(@Param("orderId") Long orderId);
}
